import exportPlugin from '../../../lib/exportPlugin'
import VideoPlugin from '../../../lib/plugin'
import { version, name } from '../package.json'

class VimeoAnalytics extends VideoPlugin {
  constructor(player, authToken) {
    super(name, version)
    this.authToken = authToken
    this.player = player
    this.metadata = {
      content: {},
      playback: { videoPlayer: 'Vimeo' }
    }
    this.mostRecentHeartbeat = 0
    this.isPaused = false
  }

  initialize() {
    const events = {
      loaded: this.retrieveMetadata,
      play: this.trackPlay,
      pause: this.trackPause,
      ended: this.trackEnded,
      timeupdate: this.trackHeartbeat
    }

    for (event in events) {
      this.registerHandler(event, events[event])
    }
  }

  // This is a helper function used to facilitate easier testing.
  registerHandler(event, handler) {
    this.player.on(event, data => {
      this.updateMetadata(data)
      handler.call(this, data)
    })
  }

  async trackPlay() {
    if (this.isPaused) {
      this.track('Video Playback Resumed', this.metadata.playback)
      this.isPaused = false
    } else {
      this.track('Video Playback Started', this.metadata.playback)
      this.track('Video Content Started', this.metadata.content)
    }
  }

  async trackEnded() {
    this.track('Video Playback Completed', this.metadata.playback)
    this.track('Video Content Completed', this.metadata.content)
  }

  // Vimeo provides time updates every 250ms while Segment documents sending heartbeats every 10s.
  // Therefore, we need to do some math to ensure we are not sending 4 heartbeat events when we reach
  // 10 second intervals.
  async trackHeartbeat() {
    const mostRecentHeartbeat = this.mostRecentHeartbeat
    const currentPosition = this.metadata.playback.position
    if (
      currentPosition !== mostRecentHeartbeat &&
      currentPosition - mostRecentHeartbeat >= 10
    ) {
      this.track('Video Content Playing', this.metadata.content)
      this.mostRecentHeartbeat = Math.floor(currentPosition)
    }
  }

  async trackPause() {
    this.isPaused = true
    this.track('Video Playback Paused', this.metadata.playback)
  }

  // retrieve static video metadata from vimeo API
  async retrieveMetadata(data) {
    const videoId = data.id
    const res = await fetch(`https://api.vimeo.com/videos/${videoId}`, {
      headers: {
        Authorization: `Bearer ${this.authToken}`
      }
    })

    if (!res.ok) {
      const err = await res.json()
      console.error('Request to Vimeo API failed with: ', err)
      return
    }

    const json = await res.json()

    this.metadata.content.title = json.name
    this.metadata.content.description = json.description
    this.metadata.content.publisher = json.user.name

    this.metadata.playback.position = 0
    this.metadata.playback.totalLength = json.duration
  }

  // update dynamic video information
  async updateMetadata(data) {
    const volume = await this.player.getVolume()
    if (volume) this.metadata.playback.sound = volume * 100
    this.metadata.playback.position = data.seconds
  }
}

export default exportPlugin(VimeoAnalytics, 'VimeoAnalytics')
